@@ -10,7 +10,7 @@ use MarpaX::Languages::ECMAScript::AST::Exceptions qw/:all/;
 
 # ABSTRACT: ECMAScript, grammars base package
 
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.011'; # VERSION
 
 #
 # Note: because this module is usually subclasses, internal methods are called
@@ -19,25 +19,15 @@ our $VERSION = '0.005'; # VERSION
 
 
 sub new {
-  my ($class, $grammar, $package, $spec) = @_;
+  my ($class, $spec) = @_;
 
-  InternalError(error => 'Missing grammar') if (! defined($grammar));
-  InternalError(error => 'Missing package name') if (! defined($package));
   InternalError(error => 'Missing ECMAScript specification') if (! defined($spec));
 
   my $self  = {
-      _content => $grammar,
-      _grammar_option => {bless_package => $package, action_object  => sprintf('%s::%s', $package, 'Actions')},
-      _recce_option => {ranking_method => 'high_rule_only'},
-      _strict => 0
+      _content        => $class->make_content($spec),
+      _grammar_option => $class->make_grammar_option($spec),
+      _recce_option   => $class->make_recce_option($spec),
   };
-  #
-  # Too painful to write MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::IsSomething
-  # so I change it on-the-fly here
-  #
-  my $characterClass = "\\p{MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::Is";
-  $self->{_content} =~ s/\\p\{Is/$characterClass/g;
-  $self->{_grammar_option}->{source} = \$self->{_content};
 
   bless($self, $class);
 
@@ -45,21 +35,31 @@ sub new {
 }
 
 
-sub strict {
-    my $self = shift;
-    if (@_) {
-	$self->{_strict} = shift;
-    }
-    return $self->{_strict};
-}
-
-
 sub content {
     my ($self) = @_;
     return $self->{_content};
 }
 
 
+sub make_content {
+    my ($class, $spec) = @_;
+
+    my $content = $class->make_grammar_content;
+
+    #
+    # Too painful to write MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::IsSomething
+    # so I change it on-the-fly here
+    #
+    if ($spec eq 'ECMAScript-262-5') {
+	$spec = 'ECMAScript_262_5';
+    }
+    my $characterClass = "\\p{MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::Is";
+    $content =~ s/\\p\{Is/$characterClass/g;
+
+    return $content;
+}
+
+
 sub extract {
     my ($self) = @_;
     my $rc = '';
@@ -76,6 +76,25 @@ sub extract {
 }
 
 
+sub make_grammar_option {
+    my ($class, $spec) = @_;
+    return {bless_package => $class->make_bless_package,
+	    source        => \$class->make_content($spec, $class->make_grammar_content)};
+}
+
+
+sub make_grammar_content {
+    my ($class) = @_;
+    return undef;
+}
+
+
+sub make_bless_package {
+    my ($class) = @_;
+    return $class;
+}
+
+
 sub grammar_option {
     my ($self) = @_;
     return $self->{_grammar_option};
@@ -88,6 +107,24 @@ sub recce_option {
 }
 
 
+sub make_recce_option {
+    my ($class, $spec) = @_;
+    return {ranking_method => $class->make_ranking_method, semantics_package => $class->make_semantics_package};
+}
+
+
+sub make_ranking_method {
+    my ($class) = @_;
+    return 'high_rule_only';
+}
+
+
+sub make_semantics_package {
+    my ($class) = @_;
+    return join('::', __PACKAGE__, 'DefaultSemanticsPackage');
+}
+
+
 sub _callback {
   my ($self, $source, $pos, $max, $impl, $callbackp, $originalErrorString, @args) = @_;
 
@@ -98,17 +135,22 @@ sub _callback {
     my $callackErrorString = $@;
     my $line_columnp;
     eval {$line_columnp = lineAndCol($impl)};
+    my $context = _context($self, $impl);
+    #
+    # Now we can destroy the recognizer
+    #
+    $impl->destroy_R;
     if (! $@) {
       if (defined($originalErrorString) && $originalErrorString) {
-        SyntaxError(error => sprintf("%s\n%s\n\n%s%s", $originalErrorString, $callackErrorString, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl)));
+        SyntaxError(error => sprintf("%s\n%s\n\n%s%s", $originalErrorString, $callackErrorString, showLineAndCol(@{$line_columnp}, $source), $context));
       } else {
-        SyntaxError(error => sprintf("%s\n\n%s%s", $callackErrorString, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl)));
+        SyntaxError(error => sprintf("%s\n\n%s%s", $callackErrorString, showLineAndCol(@{$line_columnp}, $source), $context));
       }
     } else {
       if (defined($originalErrorString) && $originalErrorString) {
-        SyntaxError(error => sprintf("%s\n%s\n%s", $originalErrorString, $callackErrorString, _context($self, $impl)));
+        SyntaxError(error => sprintf("%s\n%s\n%s", $originalErrorString, $callackErrorString, $context));
       } else {
-        SyntaxError(error => sprintf("%s\n%s", $callackErrorString, _context($self, $impl)));
+        SyntaxError(error => sprintf("%s\n%s", $callackErrorString, $context));
       }
     }
   }
@@ -129,25 +171,18 @@ sub parse {
   my $endp = $optionsp->{end};
   my $endargsp = $optionsp->{endargs} // [];
   my @endargs = @{$endargsp};
-  my $keepOriginalSource = $optionsp->{keepOriginalSource} // 1;
 
   $start //= 0;
   $length //= -1;
 
-  #
-  # This will create a new instance of the string
-  #
-  if (! $keepOriginalSource) {
-      $source .= ' ';
-  }
-
   my $pos = $start;
   my $max = length($source) - $start + $length;
   my $stop;
   my $newpos;
   #
-  # Space for an eventual last and inserted semicolon
+  # Create a recognizer
   #
+  $impl->make_R;
   #
   # Lexer can fail
   #
@@ -160,7 +195,9 @@ sub parse {
       $pos = _callback($self, $source, $pos, $max, $impl, $failurep, $@, @failureargs);
     } else {
       my $line_columnp = lineAndCol($impl);
-      SyntaxError(error => sprintf("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl)));
+      my $context = _context($self, $impl);
+      $impl->destroy_R;
+      SyntaxError(error => sprintf("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), $context));
     }
   } else {
     $pos = $newpos;
@@ -184,7 +221,9 @@ sub parse {
         $pos = _callback($self, $source, $pos, $max, $impl, $failurep, $@, @failureargs);
       } else {
         my $line_columnp = lineAndCol($impl);
-        SyntaxError(error => sprintf("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl)));
+	my $context = _context($self, $impl);
+	$impl->destroy_R;
+        SyntaxError(error => sprintf("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), $context));
       }
     } else {
       $pos = $newpos;
@@ -205,13 +244,17 @@ sub parse {
 sub value {
   my ($self, $impl) = @_;
 
-  my $rc = $impl->value() || InternalError(error => sprintf('%s', _show_last_expression($self, $impl)));
+  my $rc = $impl->value() || do {$impl->destroy_R; InternalError(error => sprintf('%s', _show_last_expression($self, $impl)))};
   if (! defined($rc)) {
+      $impl->destroy_R;
       InternalError(error => 'Undefined parse tree value');
   }
   if (defined($impl->value())) {
+      $impl->destroy_R;
       InternalError(error => 'More than one parse tree value');
   }
+  $impl->destroy_R;
+
   return ${$rc};
 }
 
@@ -297,7 +340,7 @@ MarpaX::Languages::ECMAScript::AST::Grammar::Base - ECMAScript, grammars base pa
 
 =head1 VERSION
 
-version 0.005
+version 0.011
 
 =head1 SYNOPSIS
 
@@ -317,22 +360,34 @@ This modules returns a base package for all the ECMAScript grammars written in M
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new($grammar, $package, $spec)
+=head2 new($class, $spec)
 
-Instance a new object. Takes a grammar, a package name and an ECMAScript specification as required parameters.
-
-=head2 strict($self, [$strict])
-
-Sets/Returns the strict mode of the grammar.
+Instance a new object. Takes an ECMAScript specification $spec as required parameter.
 
 =head2 content($self)
 
 Returns the content of the grammar.
 
+=head2 make_content($class, $spec)
+
+Class method that return the default content of the grammar writen for specification $spec. Grammars are typically use Posix user-defined classes without the full classname; this method is making sure full classname is used; using $spec.
+
 =head2 extract($self)
 
 Returns the part of the grammar that can be safely extracted and injected in another.
 
+=head2 make_grammar_option($class, $spec)
+
+Class method that returns default grammar options for a given ECMA specification $spec.
+
+=head2 make_grammar_content($class)
+
+Class method that returns the grammar content. This class must be overwriten by the any package providing a grammar.
+
+=head2 make_bless_package($class)
+
+Class method that returns recommended bless_package grammar options.
+
 =head2 grammar_option($self)
 
 Returns recommended option for Marpa::R2::Scanless::G->new(), returned as a reference to a hash.
@@ -341,6 +396,18 @@ Returns recommended option for Marpa::R2::Scanless::G->new(), returned as a refe
 
 Returns recommended option for Marpa::R2::Scanless::R->new(), returned as a reference to a hash.
 
+=head2 make_recce_option($class, $spec)
+
+Class method that returns default recce options for a given ECMA specification $spec.
+
+=head2 ranking_method($class)
+
+Class method that returns recommended recce ranking_method
+
+=head2 semantics_package($class)
+
+Class method that returns a default recce semantics_package, doing nothing else but a new().
+
 =head2 parse($self, $source, [$optionsp], [$start], [$length])
 
 Parse the source given as reference to a scalar, an optional reference to a options that is a hash that can contain:
@@ -371,17 +438,13 @@ End callback Code Reference. Default is undef.
 
 Reference to an array of End callback Code Reference first arguments. Default is [].
 
-=item keepOriginalSource
-
-Because of Automatic Semicolon Insertion that may happen at the end, a space is appended to a copy of the source to be parsed. If a true value, this option disable that append. Default is true.
-
 =back
 
-This method must be called as a super method by grammar using this package as a parent. $self must be a reference to a grammar instantiated via MarpaX::Languages::ECMAScript::AST::Grammar. The callback code will always be called with: per-callback arguments, $source, $pos (i.e. current position), $max (i.e. max position), $impl (i.e. a MarpaX::Languages::ECMAScript::AST::Impl instance). The default and failure callbacks must always return the new position in the stream, and raise a MarpaX::Languages::ECMAScript::AST::Exception::SyntaxError exception if there is an error. In the 'end' and 'failure' callbacks, $pos is not meaningful: this is the last position where external scanning restarted. You might want to look to the getLastLexeme() method. Output of the 'end' callback is ignored.
+This method must be called as a super method by grammar using this package as a parent. $self must be a reference to a grammar instantiated via MarpaX::Languages::ECMAScript::AST::Grammar. The callback code will always be called with: per-callback arguments, $source, $pos (i.e. current position), $max (i.e. max position), $impl (i.e. a MarpaX::Languages::ECMAScript::AST::Impl instance). The default and failure callbacks must always return the new position in the stream, and raise a MarpaX::Languages::ECMAScript::AST::Exception::SyntaxError exception if there is an error. In the 'end' and 'failure' callbacks, $pos is not meaningful: this is the last position where external scanning restarted. You might want to look to the getLastLexeme() method. Output of the 'end' callback is ignored. Please note that this method explicitely creates a recognizer using $impl->make_R(), destroyed in case of error.
 
 =head2 value($self, $impl)
 
-Return the blessed value. $impl is the recognizer instance for the grammar. Will raise an InternalError exception if there is no parse tree value, or more than one parse tree value.
+Return the parse tree (unique) value. $impl is the recognizer instance for the grammar. Will raise an InternalError exception if there is no parse tree value, or more than one parse tree value. Please note that this method explicity destroys the recognizer using $impl->destroy_R. Value itself is an AST where every string is a perl string.
 
 =head2 getLexeme($self, $lexemeHashp, $impl)
 
