@@ -11,18 +11,18 @@ use warnings FATAL => 'all';
 
 package MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program;
 use parent qw/MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Base/;
-use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program::Actions;
+use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program::Semantics;
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::RegularExpressionLiteral;
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::StringLiteral;
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::NumericLiteral;
 use MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::CharacterClasses;
 use MarpaX::Languages::ECMAScript::AST::Exceptions qw/:all/;
-use Log::Any qw/$log/;
+#use Log::Any qw/$log/;
 use SUPER;
 
 # ABSTRACT: ECMAScript-262, Edition 5, lexical program grammar written in Marpa BNF
 
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.011'; # VERSION
 
 our $WhiteSpace        = qr/(?:[\p{MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::CharacterClasses::IsWhiteSpace}])/;
 our $LineTerminator    = qr/(?:[\p{MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::CharacterClasses::IsLineTerminator}])/;
@@ -59,7 +59,7 @@ our @NullLiteral = qw/null/;
 
 our @BooleanLiteral = qw/true false/;
 
-our $grammar_source = do {local $/; <DATA>};
+our $grammar_content = do {local $/; <DATA>};
 
 #
 # It is clearer to have reserved words in an array. But for efficienvy the hash is better,
@@ -75,25 +75,24 @@ our %BooleanLiteral           = map {($_, uc($_))} @BooleanLiteral;
 #
 # ... And we inject in the grammar those that exist (FutureReservedWord do not)
 #
-$grammar_source .= "\n";
+$grammar_content .= "\n";
 # ... Priorities
-map {$grammar_source .= ":lexeme ~ <$_> priority => 1\n"} values %Keyword;
-map {$grammar_source .= ":lexeme ~ <$_> priority => 1\n"} values %NullLiteral;
-map {$grammar_source .= ":lexeme ~ <$_> priority => 1\n"} values %BooleanLiteral;
+map {$grammar_content .= ":lexeme ~ <$_> priority => 1\n"} values %Keyword;
+map {$grammar_content .= ":lexeme ~ <$_> priority => 1\n"} values %NullLiteral;
+map {$grammar_content .= ":lexeme ~ <$_> priority => 1\n"} values %BooleanLiteral;
 # ... Definition
-map {$grammar_source .= uc($_) . " ~ '$_'\n"} @Keyword;
-map {$grammar_source .= uc($_) . " ~ '$_'\n"} @NullLiteral;
-map {$grammar_source .= uc($_) . " ~ '$_'\n"} @BooleanLiteral;
+map {$grammar_content .= uc($_) . " ~ '$_'\n"} @Keyword;
+map {$grammar_content .= uc($_) . " ~ '$_'\n"} @NullLiteral;
+map {$grammar_content .= uc($_) . " ~ '$_'\n"} @BooleanLiteral;
 #
 # Injection of grammars.
 #
 our $StringLiteral = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::StringLiteral->new();
 our $RegularExpressionLiteral = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::RegularExpressionLiteral->new();
 our $NumericLiteral = MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Lexical::NumericLiteral->new();
-$grammar_source .= $StringLiteral->extract;
-$grammar_source .= $NumericLiteral->extract;
-$grammar_source .= $RegularExpressionLiteral->extract;
-
+$grammar_content .= $StringLiteral->extract;
+$grammar_content .= $NumericLiteral->extract;
+$grammar_content .= $RegularExpressionLiteral->extract;
 
 #
 # For convenience in the IDENTIFIER$ lexeme callback, we merge Keyword, FutureReservedWord, NullLiteral, BooleanLiteral into
@@ -103,18 +102,26 @@ our %ReservedWord = map {$_ => 1} (keys %Keyword, keys %FutureReservedWord, keys
 
 
 
-sub new {
+sub make_grammar_content {
     my ($class) = @_;
+    return $grammar_content;
+}
 
-    my $self = $class->SUPER($grammar_source, __PACKAGE__);
 
-    return $self;
+sub make_semantics_package {
+    my ($class) = @_;
+    return join('::', $class, 'Semantics');
 }
 
 
 sub parse {
     my ($self, $source, $impl) = @_;
+    #
+    # Because of Automatic Semicolon Insertion that may happen at the end,
+    # a space is appended to a copy of the source to be parsed.
+    #
     $self->{programCompleted} = 0;
+    $source .= ' ';
     return $self->SUPER($source, $impl,
                         {
                          callback => \&_eventCallback,
@@ -123,7 +130,6 @@ sub parse {
                          failureargs => [ $self ],
                          end => \&_endCallback,
                          endargs => [ $self ],
-			 keepOriginalSource => 0,
                         });
 }
 
@@ -134,6 +140,8 @@ sub _eventCallback {
   # $pos is the exact position where SLIF stopped because of an event
   #
   my $rc = $pos;
+  my %lastLexeme = ();
+  my $lastLexemeDoneb = 0;
 
   foreach (@{$impl->events()}) {
     my ($name) = @{$_};
@@ -163,8 +171,10 @@ sub _eventCallback {
       }
     }
     elsif ($name eq 'IDENTIFIER$') {
-	my %lastLexeme = ();
-	$self->getLastLexeme(\%lastLexeme, $impl);
+	if (! $lastLexemeDoneb) {
+	    $self->getLastLexeme(\%lastLexeme, $impl);
+	    $lastLexemeDoneb = 1;
+	}
 	if (exists($ReservedWord{$lastLexeme{value}})) {
 	    SyntaxError(error => "Identifier $lastLexeme{value} is a reserved word");
 	}
@@ -182,8 +192,10 @@ sub _eventCallback {
       #
       # In the AST, we explicitely associate the ';' to the missing semicolon
       #
-	my %lastLexeme = ();
-	$self->getLastLexeme(\%lastLexeme, $impl);
+	if (! $lastLexemeDoneb) {
+	    $self->getLastLexeme(\%lastLexeme, $impl);
+	    $lastLexemeDoneb = 1;
+	}
 	my $Slength = $self->_preSLength($source, $rc, $impl);
 	$self->_insertInvisibleSemiColon($impl, $rc, $Slength);
 	$rc += $Slength;
@@ -192,8 +204,10 @@ sub _eventCallback {
     # ^PLUSPLUS_POSTFIX, ^MINUSMINUS_POSTFIX
     # --------------------------------------
     elsif ($name eq '^PLUSPLUS_POSTFIX' || $name eq '^MINUSMINUS_POSTFIX') {
-      my %lastLexeme = ();
-      $self->getLastLexeme(\%lastLexeme, $impl);
+	if (! $lastLexemeDoneb) {
+	    $self->getLastLexeme(\%lastLexeme, $impl);
+	    $lastLexemeDoneb = 1;
+	}
       my $postLineTerminatorPos = $lastLexeme{start} + $lastLexeme{length};
       my $postLineTerminatorLength = $self->_postLineTerminatorLength($source, $postLineTerminatorPos, $impl);
       if ($postLineTerminatorLength > 0) {
@@ -463,7 +477,7 @@ MarpaX::Languages::ECMAScript::AST::Grammar::ECMAScript_262_5::Program - ECMAScr
 
 =head1 VERSION
 
-version 0.005
+version 0.011
 
 =head1 SYNOPSIS
 
@@ -483,13 +497,17 @@ This modules returns describes the ECMAScript 262, Edition 5 lexical program gra
 
 =head1 SUBROUTINES/METHODS
 
-=head2 new()
+=head2 make_grammar_content($class)
 
-Instance a new object.
+Returns the grammar. This will be injected in the Program's grammar.
 
-=head2 parse($self, $source)
+=head2 semantics_package($class)
 
-Parse the source given as $source.
+Class method that returns Program default recce semantics_package. These semantics are adding ruleId to all values, and execute eventually StringLiteral lexical grammar.
+
+=head2 parse($self, $source, $impl)
+
+Parse the source given as $source using implementation $impl.
 
 =head1 SEE ALSO
 
